import BigWorld
import ResMgr
import Math
import math
import ProjectileMover
import BattleReplay
import nations
from projectile_trajectory import computeProjectileTrajectory
from constants import SERVER_TICK_LENGTH, SHELL_TRAJECTORY_EPSILON_CLIENT, ITEM_DEFS_PATH, ARENA_GUI_TYPE
from Vehicle import Vehicle
from Avatar import PlayerAvatar
from xfw import *
from xvm_main.python.logger import *
import xvm_main.python.config as config
from xvm import utils
from helpers import dependency
from skeletons.account_helpers.settings_core import ISettingsCore
from AvatarInputHandler.AimingSystems.ArcadeAimingSystem import ArcadeAimingSystem
from AvatarInputHandler.AimingSystems.SniperAimingSystem import SniperAimingSystem
from AvatarInputHandler.AimingSystems.StrategicAimingSystem import StrategicAimingSystem
from AvatarInputHandler.gun_marker_ctrl import _CrosshairShotResults, _SHOT_RESULT
# from AvatarInputHandler.aih_constants import GUN_MARKER_TYPE
from gui.Scaleform.daapi.view.meta.CrosshairPanelContainerMeta import CrosshairPanelContainerMeta
from VehicleGunRotator import VehicleGunRotator
from gui.battle_control.controllers.consumables.ammo_ctrl import AmmoReplayPlayer
from tutorial.control.battle.functional import _StaticObjectMarker3D
import gui.Scaleform.daapi.view.battle.shared.crosshair.plugins as plug
from items import _xml
from account_helpers.settings_core.options import InterfaceScaleSetting
from string import digits
from TriggersManager import g_manager
from gui.Scaleform.daapi.view.battle.classic.stats_exchange import FragsCollectableStats

VEHICLE_CLASSES = {'mediumTank': 'MT', 'lightTank': 'LT', 'heavyTank': 'HT', 'AT-SPG': 'TD', 'SPG': 'SPG'}

SHELL_TYPES = {'armor_piercing':    '{{l10n:armor_piercing}}',
               'high_explosive':    '{{l10n:high_explosive}}',
               'armor_piercing_cr': '{{l10n:armor_piercing_cr}}',
               'armor_piercing_he': '{{l10n:armor_piercing_he}}',
               'hollow_charge':     '{{l10n:hollow_charge}}'}

COLOR_PIERCING_CHANCE = {'not_pierced':    '#E82929',
                         'little_pierced': '#E1C300',
                         'great_pierced':  '#2ED12F'}

y = 0
cameraMode = None
leftTime = None
reloadTime = None
reloadTimeClip = None
beginReloadTime = None
maxHealth = 0
health = None
quantityShells = None
quantityInClipShells = None
quantityInClipShellsMax = None
burst = None
isClip = False
currentDistance = None
vehicle = None
player = None
timeFlight = None
timeAIM = None
sphere = None
_explosionRadius = None
isDisplaySphere = False
isDownHotkey = True
armor = None
shotsDescr = None
shellType = None
shellSpeed = 0
targetDistance = None
cameraHeight = None
piercingActual = None
armorActual = None
shotResult = None
colorPiercingChance = COLOR_PIERCING_CHANCE
_typeDescriptor = {'name': None, 'vehicle': None, 'vtype': None, 'reload': None, 'visionRadius': None}
autoAim = {'name': None, 'vehicle': None, 'health': None}
f_delayHideTarget = None


isNotEvent = False
goldShell = None
_currReloadingState = 0


gold_shells = {}
xmlPath = ''

def isGoldShell(n, s):
    if n != 'icons':
        xmlCtx = (None, xmlPath + '/' + n)
        price = 'gold' in _xml.readPrice(xmlCtx, s, 'price')
        return _xml.readInt(xmlCtx, s, 'id', 0, 65535) if price else None


for nation in nations.NAMES:
    xmlPath = '%s%s%s%s' % (ITEM_DEFS_PATH, 'vehicles/', nation, '/components/shells.xml')
    gold_shells[nation] = [isGoldShell(name, subsection) for name, subsection in ResMgr.openSection(xmlPath).items() if isGoldShell(name, subsection) is not None]
ResMgr.purge(xmlPath, True)


@registerEvent(plug.AmmoPlugin, '_AmmoPlugin__onCurrentShellChanged')
def AmmoPlugin__onCurrentShellChanged(self, intCD):
    if config.get('sight/enabled', True) and isNotEvent:
        global shellType, goldShell, shellSpeed
        shells = BigWorld.player().guiSessionProvider.shared.ammo.getOrderedShellsLayout()
        for shell in shells:
            if shell[0] == intCD:
                shellType = config.get('sight/shellType', SHELL_TYPES).get(shell[1].kind.lower(), None)
                id = shell[1].id
                for shot in shotsDescr:
                    if shot.shell.id == id:
                        shellSpeed = int(shot.speed)
                        break
                goldShell = 'gold' if id[1] in gold_shells[nations.NAMES[id[0]]] else None
                break
        as_event('ON_AMMO_CHANGED')


@registerEvent(plug.ShotResultIndicatorPlugin, '_ShotResultIndicatorPlugin__onGunMarkerStateChanged')
def __onGunMarkerStateChanged(self, markerType, position, dir, collision):
    if config.get('sight/enabled', True) and isNotEvent:
        if not self._ShotResultIndicatorPlugin__isEnabled:
             self._ShotResultIndicatorPlugin__shotResultResolver.getShotResult(position, collision, dir, excludeTeam=self._ShotResultIndicatorPlugin__playerTeam)


def reloadTimer():
    global leftTime
    if leftTime is None:
        return
    if leftTime > 0.0:
        leftTime = max(0.0, beginReloadTime - BigWorld.time())
        if leftTime <= 0.0:
            leftTime = 0.0
        else:
            BigWorld.callback(0.1, reloadTimer)
    as_event('ON_RELOAD')


def reloading(duration, baseTime, startTime):
    global leftTime, reloadTime, beginReloadTime, reloadTimeClip, _currReloadingState
    if duration > 0.0:
        if leftTime != 0.0:
            beginReloadTime = BigWorld.time() + duration
            leftTime = duration
        else:
            beginReloadTime = BigWorld.time() + duration - startTime
            leftTime = duration - startTime
        if baseTime != 0.0:
            if (baseTime != 0.0) and ((reloadTimeClip is None) or (int(reloadTimeClip) != int(baseTime))):
                reloadTime = baseTime
        BigWorld.callback(0.1, reloadTimer)
    else:
        if (baseTime != 0.0) and ((reloadTimeClip is None) or (int(reloadTimeClip) != int(baseTime))):
            reloadTime = baseTime
        leftTime = 0.0
    explosionRadius()
    as_event('ON_RELOAD')


@registerEvent(AmmoReplayPlayer, 'setGunReloadTime')
def AmmoReplayPlayer_setGunReloadTime(self, timeLeft, baseTime):
    if config.get('sight/enabled', True) and isNotEvent:
        reloading(timeLeft, baseTime, 0.0)


@registerEvent(ArcadeAimingSystem, 'enable')
def ArcadeAimingSystem_enable(self, targetPos, turretYaw=None, gunPitch=None):
    if config.get('sight/enabled', True) and isNotEvent:
        global y, cameraMode
        y = - BigWorld.screenHeight() * 0.0775
        cameraMode = 'arc'
        as_event('ON_CAMERA_MODE')


@overrideMethod(CrosshairPanelContainerMeta, 'as_setAmmoStockS')
def CrosshairPanelContainerMeta_as_setAmmoStockS(base, self, quantity, quantityInClip, isLow, clipState, clipReloaded):
    if config.get('sight/enabled', True) and isNotEvent:
        global quantityShells, quantityInClipShells
        quantityShells = max(quantity, 0)
        quantityInClipShells = max(quantityInClip, 0) if isClip else None
        if config.get('sight/removeQuantityShells', False):
            base(self, -1, quantityInClip, isLow, clipState, clipReloaded)
        else:
            base(self, quantity, quantityInClip, isLow, clipState, clipReloaded)
        as_event('ON_AMMO_CHANGED')
    else:
        return base(self, quantity, quantityInClip, isLow, clipState, clipReloaded)


@overrideMethod(CrosshairPanelContainerMeta, 'as_setDistanceS')
def CrosshairPanelContainerMeta_as_setDistanceS(base, self, dist):
    if config.get('sight/enabled', True) and isNotEvent:
        global targetDistance
        i = len(dist) - 1
        while dist[i] not in digits:
            i -= 1
        targetDistance = int(dist[:i+1])
        if not config.get('sight/removeDistance', False):
            return base(self, dist)
    else:
        return base(self, dist)


@registerEvent(CrosshairPanelContainerMeta, 'as_setReloadingS')
def CrosshairPanelContainerMeta_as_setReloadingS(self, duration, baseTime, startTime, isReloading):
    if config.get('sight/enabled', True) and isNotEvent:
        reloading(duration, baseTime, startTime)


@overrideClassMethod(_CrosshairShotResults, 'getShotResult')
def _CrosshairShotResults_getShotResult(base, cls, hitPoint, collision, dir, excludeTeam=0):
    # log('getShotResult')
    if config.get('sight/enabled', True) and isNotEvent:
        def updateLabel():
            if (old_armorActual != armorActual) or (old_piercingActual != piercingActual):
                # log('ON_CALC_ARMOR')
                as_event('ON_CALC_ARMOR')

        global piercingActual, armorActual, shotResult
        keyColor = [None, 'not_pierced', 'little_pierced', 'great_pierced']
        old_piercingActual = piercingActual
        old_armorActual = armorActual
        piercingActual = None
        armorActual = None
        shotResult = keyColor[0]
        if collision is None:
            updateLabel()
            return _SHOT_RESULT.UNDEFINED
        else:
            entity = collision.entity
            if entity.__class__.__name__ != 'Vehicle':
                updateLabel()
                return _SHOT_RESULT.UNDEFINED
            if entity.health <= 0 or entity.publicInfo['team'] == excludeTeam:
                updateLabel()
                return _SHOT_RESULT.UNDEFINED
            player = BigWorld.player()
            if player is None:
                updateLabel()
                return _SHOT_RESULT.UNDEFINED
            vDesc = player.getVehicleDescriptor()
            shell = vDesc.shot.shell
            caliber = shell.caliber
            shellKind = shell.kind
            ppDesc = vDesc.shot.piercingPower
            maxDist = vDesc.shot.maxDistance
            dist = (hitPoint - player.getOwnVehiclePosition()).length
            piercingPower = cls._computePiercingPowerAtDist(ppDesc, dist, maxDist)
            fullPiercingPower = piercingPower
            minPP, maxPP = cls._computePiercingPowerRandomization(shell)
            result = _SHOT_RESULT.NOT_PIERCED
            isJet = False
            jetStartDist = None
            ignoredMaterials = set()
            collisionsDetails = cls._getAllCollisionDetails(hitPoint, dir, entity)
            if collisionsDetails is None:
                updateLabel()
                return _SHOT_RESULT.UNDEFINED
            for cDetails in collisionsDetails:
                if isJet:
                    jetDist = cDetails.dist - jetStartDist
                    if jetDist > 0.0:
                        piercingPower *= 1.0 - jetDist * cls._SHELL_EXTRA_DATA[shellKind].jetLossPPByDist
                if cDetails.matInfo is None:
                    result = cls._CRIT_ONLY_SHOT_RESULT
                else:
                    matInfo = cDetails.matInfo
                    if (cDetails.compName, matInfo.kind) in ignoredMaterials:
                        continue
                    hitAngleCos = cDetails.hitAngleCos if matInfo.useHitAngle else 1.0
                    if not isJet and cls._shouldRicochet(shellKind, hitAngleCos, matInfo, caliber):
                        break
                    piercingPercent = 1000.0
                    if piercingPower > 0.0:
                        penetrationArmor = cls._computePenetrationArmor(shellKind, hitAngleCos, matInfo, caliber)
                        piercingPercent = 100.0 + (penetrationArmor - piercingPower) / fullPiercingPower * 100.0
                        piercingActual = int(piercingPower)
                        armorActual = int(penetrationArmor)
                        piercingPower -= penetrationArmor
                        # log('penetrationArmor = %s     piercingPercent = %s    piercingPower = %s' % (penetrationArmor, piercingPercent, piercingPower))
                    if matInfo.vehicleDamageFactor:
                        if minPP < piercingPercent < maxPP:
                            result = _SHOT_RESULT.LITTLE_PIERCED
                        elif piercingPercent <= minPP:
                            result = _SHOT_RESULT.GREAT_PIERCED
                        break
                    elif matInfo.extra:
                        if piercingPercent <= maxPP:
                            result = cls._CRIT_ONLY_SHOT_RESULT
                    if matInfo.collideOnceOnly:
                        ignoredMaterials.add((cDetails.compName, matInfo.kind))
                if piercingPower <= 0.0:
                    break
                if cls._SHELL_EXTRA_DATA[shellKind].jetLossPPByDist > 0.0:
                    isJet = True
                    mInfo = cDetails.matInfo
                    armor = mInfo.armor if mInfo is not None else 0.0
                    jetStartDist = cDetails.dist + armor * 0.001
            shotResult = keyColor[result]
            updateLabel()
            return result
    else:
        return base(hitPoint, collision, dir, excludeTeam)


def explosionRadius():
    global _explosionRadius
    if vehicle is None or not hasattr(vehicle, 'typeDescriptor'):
        return
    _type = vehicle.typeDescriptor.shot.shell.type
    _explosionRadius = _type.explosionRadius if hasattr(_type, 'explosionRadius') else None


@registerEvent(FragsCollectableStats, 'addVehicleStatusUpdate')
def FragsCollectableStats_addVehicleStatusUpdate(self, vInfoVO):
    if config.get('sight/enabled', True) and isNotEvent:
        if (not vInfoVO.isAlive()) and (vehicle is not None) and (vehicle.id == vInfoVO.vehicleID):
            setValueDead()


@registerEvent(PlayerAvatar, 'handleKey')
def handleKey(self, isDown, key, mods):
    if config.get('sight/enabled', True) and isNotEvent:
        global isDownHotkey
        hotkey = config.get('sight/sphereDispersion/hotkey', None)
        if hotkey is not None and hotkey['enabled'] and (key == hotkey['keyCode']):
            if isDown:
                if hotkey['onHold']:
                    if not isDownHotkey:
                        isDownHotkey = True
                else:
                    isDownHotkey = not isDownHotkey
            else:
                if hotkey['onHold']:
                    if isDownHotkey:
                        isDownHotkey = False


def typeDescriptorClear():
    global _typeDescriptor, targetDistance
    targetDistance = None
    _typeDescriptor = {'name': None, 'vehicle': None, 'vtype': None, 'reload': None,
                       'visionRadius': None}


def delayHideTarget():
    global f_delayHideTarget
    f_delayHideTarget = None
    typeDescriptorClear()
    as_event('ON_TARGET')


@registerEvent(PlayerAvatar, 'targetBlur')
def PlayerAvatar_targetBlur(self, prevEntity):
    global f_delayHideTarget
    if config.get('sight/enabled', True) and isNotEvent:
        if prevEntity in self._PlayerAvatar__vehicles:
            delay = config.get('sight/delayHideTarget', 0)
            f_delayHideTarget = BigWorld.callback(delay, delayHideTarget)


@registerEvent(PlayerAvatar, 'targetFocus')
def PlayerAvatar_targetFocus(self, entity):
    if config.get('sight/enabled', True) and isNotEvent:
        global _typeDescriptor, f_delayHideTarget
        if entity in self._PlayerAvatar__vehicles:
            if f_delayHideTarget is not None:
                BigWorld.cancelCallback(f_delayHideTarget)
            _type = entity.typeDescriptor.type
            _gun = entity.typeDescriptor.gun
            _miscAttrs = entity.typeDescriptor.miscAttrs
            _turret = entity.typeDescriptor.turret
            crewLevelIncrease = 0.0043 * _miscAttrs.get('crewLevelIncrease', 0)
            _typeDescriptor['vehicle'] = _type.shortUserString
            _typeDescriptor['name'] = entity.publicInfo.name
            _typeDescriptor['reload'] = _gun.reloadTime * _miscAttrs.get('gunReloadTimeFactor', 1) / (1.0695 + crewLevelIncrease)
            _typeDescriptor['visionRadius'] = _turret.circularVisionRadius * _miscAttrs.get('circularVisionRadiusFactor', 1) / (1.0434 + crewLevelIncrease)
            vehClass = VEHICLE_CLASSES[list(_type.tags.intersection(VEHICLE_CLASSES.keys()))[0]]
            _typeDescriptor['vtype'] = config.get('texts/vtype/' + vehClass)
            # _typeDescriptor['piercingActual'] = piercingActual(entity.position)
            as_event('ON_TARGET')


@overrideMethod(plug, '_makeSettingsVO')
def plugins_makeSettingsVO(base, settingsCore, *keys):
    data = base(settingsCore, *keys)
    if config.get('sight/enabled', True) and isNotEvent:
        for mode in data:
            if config.get('sight/removeCentralMarker', False) and ('centerAlphaValue' in data[mode]):
                data[mode]['centerAlphaValue'] = 0
            if config.get('sight/removeIndicator', False) and ('netAlphaValue' in data[mode]):
                data[mode]['netAlphaValue'] = 0
            if config.get('sight/removeLoad', False) and ('reloaderAlphaValue' in data[mode]):
                data[mode]['reloaderAlphaValue'] = 0
            if config.get('sight/removeCondition', False) and ('conditionAlphaValue' in data[mode]):
                data[mode]['conditionAlphaValue'] = 0
            if config.get('sight/removeContainers', False) and ('cassetteAlphaValue' in data[mode]):
                data[mode]['cassetteAlphaValue'] = 0
            if config.get('sight/removeLoadingTimer', False) and ('reloaderTimerAlphaValue' in data[mode]):
                data[mode]['reloaderTimerAlphaValue'] = 0
            if config.get('sight/removeZoomIndicator', False) and ('zoomIndicatorAlphaValue' in data[mode]):
                data[mode]['zoomIndicatorAlphaValue'] = 0
    return data


@registerEvent(InterfaceScaleSetting, 'setSystemValue')
def InterfaceScaleSetting_setSystemValue(self, value):
    if config.get('sight/enabled', True) and isNotEvent:
        global y
        y = - BigWorld.screenHeight() * 0.0775 if cameraMode == 'arc' else 0
        as_event('ON_CAMERA_MODE')


@registerEvent(SniperAimingSystem, 'enable')
def SniperAimingSystem_enable(self, targetPos, playerGunMatFunction):
    if config.get('sight/enabled', True) and isNotEvent:
        global y, cameraMode
        y = 0
        cameraMode = 'sn'
        as_event('ON_CAMERA_MODE')


@registerEvent(StrategicAimingSystem, 'enable')
def StrategicAimingSystem_enable(self, targetPos):
    if config.get('sight/enabled', True) and isNotEvent:
        global y, cameraMode
        y = 0
        cameraMode = 'str'
        as_event('ON_CAMERA_MODE')


@registerEvent(PlayerAvatar, 'onLockTarget')
def onLockTarget(self, state, playVoiceNotifications):
    if config.get('sight/enabled', True) and isNotEvent:
        global autoAim
        target = BigWorld.target()
        if (state == 1) and target is not None:
            autoAim['vehicle'] = target.typeDescriptor.type.shortUserString
            autoAim['name'] = target.publicInfo.name
            autoAim['health'] = target.health
        else:
            autoAim = {'name': None, 'vehicle': None, 'health': None}
        as_event('ON_AUTO_AIM')


@registerEvent(Vehicle, 'onEnterWorld')
def Vehicle_onEnterWorld(self, prereqs):
    if self.isPlayerVehicle and config.get('sight/enabled', True):
        global y, cameraMode, leftTime, reloadTime, maxHealth, health, quantityShells, quantityInClipShells, isClip, isNotEvent
        global vehicle, currentDistance, player, timeFlight, reloadTimeClip, quantityInClipShellsMax, timeAIM
        global burst, _explosionRadius, _typeDescriptor, isDisplaySphere, isDownHotkey, armor, shellType, shellSpeed
        global shotsDescr, targetDistance, autoAim, f_delayHideTarget, cameraHeight, piercingActual, armorActual, shotResult
        global colorPiercingChance
        _explosionRadius = None
        quantityShells = None
        quantityInClipShells = None
        leftTime = None
        reloadTime = None
        currentDistance = None
        timeFlight = None
        timeAIM = None
        armor = None
        targetDistance = None
        cameraHeight = None
        typeDescriptorClear()
        autoAim = {'name': None, 'vehicle': None, 'health': None}
        piercingActual = None
        armorActual = None
        shotResult = None
        colorPiercingChance = config.get('sight/c_piercingChance', COLOR_PIERCING_CHANCE)
        f_delayHideTarget = None
        player = BigWorld.player()
        # isNotEvent = player.arenaGuiType not in [ARENA_GUI_TYPE.EVENT_BATTLES, ARENA_GUI_TYPE.EVENT_BATTLES_2]
        isNotEvent = True
        if isNotEvent:
            vehicle = self
            td = self.typeDescriptor
            gun = td.gun
            maxHealth = td.maxHealth
            health = self.health
            y = - BigWorld.screenHeight() * 0.0775
            cameraMode = 'arc'
            quantityInClipShellsMax = gun.clip[0]
            reloadTimeClip = gun.clip[1] if isClip else None
            burst = gun.burst[0]
            isClip = quantityInClipShellsMax > 1
            _type = td.type
            shotsDescr = gun.shots
            shot = td.shot
            shellSpeed = int(shot.speed)
            shellType = config.get('sight/shellType', SHELL_TYPES).get(shot.shell.kind.lower(), None)
            vehClass = VEHICLE_CLASSES[list(_type.tags.intersection(VEHICLE_CLASSES.keys()))[0]]
            isDisplaySphere = config.get('sight/sphereDispersion/enabled', False) and (vehClass in config.get('sight/sphereDispersion/vtype', ['SPG']))
            isDownHotkey = not config.get('sight/sphereDispersion/hotkey/enabled', False)

def setValueDead():
    global leftTime, reloadTime, maxHealth, health, quantityShells, quantityInClipShells, isClip
    global currentDistance, timeFlight, reloadTimeClip, quantityInClipShellsMax, timeAIM
    global burst, _explosionRadius, isDisplaySphere, isDownHotkey, armor, shellType, shellSpeed
    global targetDistance, autoAim, f_delayHideTarget, cameraHeight, piercingActual, armorActual, shotResult
    _explosionRadius = None
    maxHealth = 0
    health = None
    quantityShells = None
    quantityInClipShells = None
    quantityInClipShellsMax = None
    burst = None
    isClip = quantityInClipShellsMax > 1
    leftTime = None
    reloadTime = None
    reloadTimeClip = None
    currentDistance = None
    timeFlight = None
    timeAIM = None
    armor = None
    shellSpeed = None
    shellType = None
    isDisplaySphere = False
    isDownHotkey = False
    targetDistance = None
    cameraHeight = None
    typeDescriptorClear()
    autoAim = {'name': None, 'vehicle': None, 'health': None}
    piercingActual = None
    armorActual = None
    shotResult = None
    f_delayHideTarget = None
    as_event('ON_AUTO_AIM')
    as_event('ON_RELOAD')
    as_event('ON_PLAYER_HEALTH')
    as_event('ON_AMMO_CHANGED')
    as_event('ON_MARKER_POSITION')
    as_event('ON_TARGET')
    as_event('ON_CALC_ARMOR')


@registerEvent(Vehicle, 'onHealthChanged')
def onHealthChanged(self, newHealth, attackerID, attackReasonID):
    if self.isPlayerVehicle and config.get('sight/enabled', True) and isNotEvent:
        global health, autoAim, piercingActual, armorActual, shotResult
        health = max(0, newHealth)
        if health == 0:
            # setValueDead()
            if sphere in BigWorld.models():
                BigWorld.delModel(sphere)
        as_event('ON_PLAYER_HEALTH')


def update_sphere(position):
    global sphere
    if sphere is None:
        sphere = BigWorld.Model('objects/misc/bbox/sphere1.model')
    elif sphere in BigWorld.models():
        BigWorld.delModel(sphere)
    if (_explosionRadius is not None) and (health > 0) and isDownHotkey:
        sphere.position = position
        sphere.scale = Math.Vector3(_explosionRadius, _explosionRadius, _explosionRadius)
        BigWorld.addModel(sphere)


@overrideMethod(VehicleGunRotator, '_VehicleGunRotator__getGunMarkerPosition')
def _VehicleGunRotator__getGunMarkerPosition(base, self, shotPos, shotVec, dispersionAngles):
    if config.get('sight/enabled', True) and isNotEvent:
        global timeFlight, currentDistance, timeAIM, vehicle, cameraHeight
        shotDescr = self._VehicleGunRotator__avatar.getVehicleDescriptor().shot
        gravity = Math.Vector3(0.0, -shotDescr.gravity, 0.0)
        maxDist = shotDescr.maxDistance
        testStartPoint = shotPos
        testEndPoint = shotPos + shotVec * 10000.0
        testVehicleID = self.getAttachedVehicleID()
        vehicle = player.getVehicleAttached()
        testEntities = ProjectileMover.getCollidableEntities((testVehicleID,), testStartPoint, testEndPoint)
        collideVehiclesAndStaticScene = ProjectileMover.collideVehiclesAndStaticScene
        collideWithSpaceBB = self._VehicleGunRotator__avatar.arena.collideWithSpaceBB
        prevPos = shotPos
        prevVelocity = shotVec
        dt = 0.0
        maxDistCheckFlag = False
        while True:
            dt += SERVER_TICK_LENGTH
            checkPoints = computeProjectileTrajectory(prevPos, prevVelocity, gravity, SERVER_TICK_LENGTH, SHELL_TRAJECTORY_EPSILON_CLIENT)
            prevCheckPoint = prevPos
            bBreak = False
            for curCheckPoint in checkPoints:
                testRes = collideVehiclesAndStaticScene(prevCheckPoint, curCheckPoint, testEntities)
                if testRes is not None:
                    collData = testRes[1]
                    if collData is not None and not collData.isVehicle():
                        collData = None
                    dir = testRes[0] - prevCheckPoint
                    endPos = testRes[0]
                    timeFlight = dt if (curCheckPoint - prevCheckPoint).length < dir.length * 2 else dt - SERVER_TICK_LENGTH
                    bBreak = True
                    break
                pos = collideWithSpaceBB(prevCheckPoint, curCheckPoint)
                if pos is not None:
                    collData = None
                    maxDistCheckFlag = True
                    dir = pos - prevCheckPoint
                    timeFlight = dt if (curCheckPoint - prevCheckPoint).length < dir.length * 2 else dt - SERVER_TICK_LENGTH
                    endPos = pos
                    bBreak = True
                    break
                prevCheckPoint = curCheckPoint
            if bBreak:
                break
            prevPos = shotPos + shotVec.scale(dt) + gravity.scale(dt * dt * 0.5)
            prevVelocity = shotVec + gravity.scale(dt)

        dir.normalise()
        cameraHeight = BigWorld.camera().position.y - endPos.y
        distance = (endPos - shotPos).length
        markerDiameter = 2.0 * distance * dispersionAngles[0]
        idealMarkerDiameter = 2.0 * distance * dispersionAngles[1]
        if maxDistCheckFlag:
            if endPos.distTo(shotPos) >= maxDist:
                dir = endPos - shotPos
                dir.normalise()
                endPos = shotPos + dir.scale(maxDist)
                distance = maxDist
                markerDiameter = 2.0 * distance * dispersionAngles[0]
                idealMarkerDiameter = 2.0 * distance * dispersionAngles[1]
        replayCtrl = BattleReplay.g_replayCtrl
        if replayCtrl.isPlaying and replayCtrl.isClientReady:
            markerDiameter, endPos, dir = replayCtrl.getGunMarkerParams(endPos, dir)
        currentDistance = distance
        speedValue = vehicle.speedInfo.value
        show_timeAIM = (speedValue[0] < 0.5) and (speedValue[1] < 0.02) and (speedValue[1] > -0.02) and (-0.5 < speedValue[0])
        if (dispersionAngles[0] >= dispersionAngles[1]):
            if show_timeAIM:
                lg = 10 * math.log(dispersionAngles[0] / dispersionAngles[1])
                timeAIM = lg if lg > 0.01 else 0
            else:
                timeAIM = None
        as_event('ON_MARKER_POSITION')
        if isDisplaySphere:
            update_sphere(endPos)
        return endPos, dir, markerDiameter, idealMarkerDiameter, collData
    else:
        return base(self, shotPos, shotVec, dispersionAngles)


@xvm.export('sight.cameraMode', deterministic=False)
def sight_cameraMode():
    return cameraMode


@xvm.export('sight.leftTime', deterministic=False)
def sight_leftTime(norm=None):
    if reloadTime is not None:
        if norm is None:
            return leftTime
        else:
            return 0 if reloadTime == 0.0 else int((reloadTime - leftTime) * norm // reloadTime)

@xvm.export('sight.reloadPercent', deterministic=False)
def sight_reloadPercent():
    if reloadTime is not None:
        return 0 if reloadTime == 0 else int((reloadTime - leftTime) * 100 // reloadTime)


@xvm.export('sight.reloadTime', deterministic=False)
def sight_reloadTime():
    return reloadTime


@xvm.export('sight.reloadTimeClip', deterministic=False)
def sight_reloadTimeClip():
    return reloadTimeClip


@xvm.export('sight.sight_y', deterministic=False)
def sight_sight_y(shift=0):
    return y + int(shift)


@xvm.export('sight.quantityShells', deterministic=False)
def sight_quantityShells():
    return quantityShells


@xvm.export('sight.quantityInClipShells', deterministic=False)
def sight_quantityInClipShells():
    return quantityInClipShells


@xvm.export('sight.quantityInClipShellsMax', deterministic=False)
def sight_quantityInClipShellsMax():
    return quantityInClipShellsMax


@xvm.export('sight.burst', deterministic=False)
def sight_burst():
    return burst


@xvm.export('sight.health', deterministic=False)
def sight_health(_health=None):
    if (health is not None) and (health > 0):
        return health if _health is None or (maxHealth == 0) else int(_health * health // maxHealth)


@xvm.export('sight.distance', deterministic=False)
def sight_distance():
    return currentDistance


@xvm.export('sight.timeFlight', deterministic=False)
def sight_timeFlight():
    return timeFlight


@xvm.export('sight.timeAIM', deterministic=False)
def sight_timeAIM():
    return timeAIM


@xvm.export('sight.vehicleTarget', deterministic=False)
def sight_vehicle():
    return _typeDescriptor.get('vehicle', None)


@xvm.export('sight.nameTarget', deterministic=False)
def sight_name():
    return _typeDescriptor.get('name', None)


@xvm.export('sight.vtypeTarget', deterministic=False)
def sight_vtype():
    return _typeDescriptor.get('vtype', None)


@xvm.export('sight.piercingActual', deterministic=False)
def sight_piercingActual():
    return piercingActual


@xvm.export('sight.armorActual', deterministic=False)
def sight_piercingActual():
    return armorActual


@xvm.export('sight.c_piercingChance', deterministic=False)
def sight_piercingActual():
    return colorPiercingChance.get(shotResult, None)


@xvm.export('sight.distanceTarget', deterministic=False)
def sight_distanceTarget():
    return targetDistance


@xvm.export('sight.reloadTarget', deterministic=False)
def sight_piercingActual():
    return _typeDescriptor.get('reload')


@xvm.export('sight.visionRadiusTarget', deterministic=False)
def sight_piercingActual():
    return _typeDescriptor.get('visionRadius')


@xvm.export('sight.armor', deterministic=False)
def sight_armor():
    return armor


@xvm.export('sight.shellType', deterministic=False)
def sight_shellType():
    return shellType


@xvm.export('sight.shellSpeed', deterministic=False)
def sight_shellSpeed():
    return shellSpeed


@xvm.export('sight.goldShell', deterministic=False)
def sight_goldShell():
    return goldShell


@xvm.export('sight.autoAimName', deterministic=False)
def sight_autoAimName():
    return autoAim.get('name', None)


@xvm.export('sight.autoAimVehicle', deterministic=False)
def sight_autoAimVehicle():
    return autoAim.get('vehicle', None)


@xvm.export('sight.autoAimHealth', deterministic=False)
def sight_autoAimVehicle():
    return autoAim.get('health', None)


@xvm.export('sight.cameraHeight', deterministic=False)
def sight_cameraHeight():
    return cameraHeight


@xvm.export('sight.dynamic_colorRGB')
def smooth_transition_colorRGB(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('RGB', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorRBG')
def smooth_transition_colorRBG(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('RBG', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorGRB')
def smooth_transition_colorGRB(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('GRB', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorGBR')
def smooth_transition_colorGBR(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('GBR', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorBRG')
def smooth_transition_colorBRG(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('BRG', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorBGR')
def smooth_transition_colorBGR(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
